#include <stdio.h>
#include <stdlib.h> // abs()

/* b/a の分数を表す型 (a>0) */
typedef struct {
    int b; // 分子
    int a; // 分母
} frac_t;

/* xとyの最大公約数を返す (x,y>=0 かつ x+y>0 とする) */
int gcd(int x, int y) {
    while (y != 0) { // ユークリッドの互除法
        int t = x % y;  x = y;  y = t;
    }
    return x;
}

/* 約分（x を約分して返す） */
frac_t frac_reduce(frac_t x) {
    int d = gcd(abs(x.b), x.a);
    x.b = x.b / d;
    x.a = x.a / d;
    return x;
}

/* 和（x+y を約分してから返す） */
frac_t frac_add(frac_t x, frac_t y) {
    frac_t ret;
    ret.b = x.b * y.a + x.a * y.b;
    ret.a = x.a * y.a;
    return frac_reduce(ret); // 約分を利用
}

/* 差（x-y を約分してから返す） */
frac_t frac_sub(frac_t x, frac_t y) {
    y.b = -y.b;
    return frac_add(x, y); // 和を利用
}

void frac_print(frac_t x) {
    printf("%d/%d", x.b, x.a);
}

int main(void) {
    frac_t x = { 7, 12 }; // 7/12
    frac_t y = { 1, 4 };  // 1/4

    printf("add "); frac_print(frac_add(x,y)); printf("\n"); // 5/6
    printf("sub "); frac_print(frac_sub(x,y)); printf("\n"); // 1/3
    printf("0   "); frac_print(frac_sub(x,x)); printf("\n"); // 0/1
    return 0;
}
